package dk.nsp.idsas.ws;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.time.Instant;
import java.time.ZoneId;
import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;
import java.util.stream.Collectors;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.LogManager;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Value;

public abstract class AbstractDksServlet extends AbstractServlet {
	private static final String RESOURCE_NAME = "/dksconfig/idsas.xml";
	private static final String DATETIME_FORMAT = "yyyy-MM-dd'T'HH:mm:ss'Z'";
	private static final Logger logger = LogManager.getLogger(AbstractDksServlet.class);

	@Value("${dcc.endpoint}")
	private String dccEndpoint;

	@Override
	public void doGet(HttpServletRequest request, HttpServletResponse response) throws IOException {
		logger.trace("dksconfig request");
		response.setContentType("application/xml");
		response.setStatus(200);
		response.addHeader("Cache-Control", "no-cache");
		String resp = dksConfig();
		response.getWriter().print(resp);
	}

	private String dksConfig() throws IOException {
		URL url = getClass().getResource(RESOURCE_NAME);
		ZonedDateTime zdt = ZonedDateTime.ofInstant(Instant.ofEpochMilli(url.openConnection().getLastModified()), ZoneId.systemDefault());
		DateTimeFormatter dateTimeFormatter = DateTimeFormatter.ofPattern(DATETIME_FORMAT);
		String timestamp = dateTimeFormatter.format(zdt);

		try (InputStream in = getClass().getResourceAsStream(RESOURCE_NAME);
			 BufferedReader reader = new BufferedReader(new InputStreamReader(in))) {
			String out = reader.lines().collect(Collectors.joining());
			return getStringWithResolvedVariables(new StringBuilder(out), timestamp);
		}
	}

	private String getStringWithResolvedVariables(StringBuilder template, String timestamp) {
		int timestampIndex = template.indexOf("TIMESTAMP");
		template.replace(timestampIndex, timestampIndex + 9, timestamp);
		return template.toString().replace("ENDPOINT_PLACEHOLDER", dccEndpoint);
	}
}
