package dk.nsp.idsas.dao;

import dk.nsp.idsas.dao.entity.BlurringEntity;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import javax.sql.DataSource;
import java.time.*;
import java.util.List;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class BlurringDaoTest extends AbstractDaoTest {
    @Autowired
    DataSource dataSource;

    private BlurringDao blurringDao;

    private static final BlurringEntity BLURRING1 = new BlurringEntity("1111111111", "CPR", "1111", "CVR", instantFrom(3000, 1, 1, 0, 0));
    private static final BlurringEntity BLURRING3 = new BlurringEntity("2222222222", "CPR", "3333", "CVR", instantFrom(3000, 1, 1, 0, 0));
    private static final BlurringEntity BLURRING4 = new BlurringEntity("2222222222", "CPR", "4444", "CVR", instantFrom(3000, 1, 1, 0, 0));

    @Before
    public void setUp() {
        blurringDao = new BlurringDao(dataSource);
    }

    @Test
    public void getActiveBlurrings_WhenOneMatchingActiveBlurringIsFound_ReturnsBlurring() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("1111111111", "CPR");

        assertEquals(1, blurrings.size());
        assertEquals(BLURRING1, blurrings.get(0));
    }

    @Test
    public void getActiveBlurrings_WhenTwoMatchingActiveBlurringAreFound_ReturnsBlurrings() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("2222222222", "CPR");

        assertEquals(2, blurrings.size());
        assertTrue(blurrings.contains(BLURRING3));
        assertTrue(blurrings.contains(BLURRING4));
    }

    @Test
    public void getActiveBlurrings_WhenMatchingExpiredBlurringIsFound_ReturnsEmptyList() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("3333333333", "CPR");

        assertEquals(0, blurrings.size());
    }

    @Test
    public void getActiveBlurrings_WhenNoMatchingBlurringIsFound_ReturnsEmptyList() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("4444444444", "CPR");

        assertEquals(0, blurrings.size());
    }

    @Test
    public void getActiveBlurrings_AfterInsert_ReturnsBlurring() {
        BlurringEntity blurringEntity = new BlurringEntity("0000000000", "CPR", "5555", "CVR", instantFrom(3000, 1, 1, 0, 0));

        blurringDao.insert(blurringEntity);
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings(blurringEntity.getPatientId(), blurringEntity.getPatientIdType());

        assertEquals(1, blurrings.size());
        assertEquals(blurringEntity, blurrings.get(0));
    }

    @Test
    public void getActiveBlurrings_WhenTwoBlurringsExistsForSamePatientAndOrg_ReturnsOnlyOne() {
        List<BlurringEntity> blurrings = blurringDao.getActiveBlurrings("5555555555", "CPR");

        assertEquals(1, blurrings.size());
    }

    private static Instant instantFrom(int year, int month, int dayOfMonth, int hour, int minute) {
        return LocalDateTime.of(year, month, dayOfMonth, hour, minute).atZone(ZoneId.systemDefault()).toInstant();
    }
}