package dk.nsp.idsas.dao;

import dk.nsp.idsas.util.ErrorMessages;
import org.junit.Before;
import org.junit.Test;
import org.springframework.beans.factory.annotation.Autowired;

import javax.sql.DataSource;

import static org.junit.Assert.*;

public class SaltDaoTest extends AbstractDaoTest {
    @Autowired
    DataSource dataSource;

    private SaltDao saltDao;

    @Before
    public void setUp() {
        saltDao = new SaltDao(dataSource);
    }

    @Test
    public void getLatestSalt_WhenSaltTableIsEmpty_ThrowsException() {
        RuntimeException e = assertThrows(RuntimeException.class, () -> saltDao.getLatestSalt());

        assertEquals(ErrorMessages.noSaltError(), e.getMessage());
    }

    @Test
    public void getLatestSalt_AfterInsert_ReturnsInsertedSalt() {
        byte[] salt = {0};
        saltDao.insert(salt);

        byte[] latestSalt = saltDao.getLatestSalt();

        assertArrayEquals(salt, latestSalt);
    }

    @Test
    public void getLatestSalt_AfterMultipleInserts_ReturnsLatestSalt() {
        byte[] salt1 = {0};
        byte[] salt2 = {1};
        saltDao.insert(salt1);
        saltDao.insert(salt2);

        byte[] latestSalt = saltDao.getLatestSalt();

        assertArrayEquals(salt2, latestSalt);
    }
}