package dk.nsp.idsas.util.health;

import dk.nsp.idsas.dao.AlivenessDao;
import org.apache.commons.httpclient.HttpStatus;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertTrue;

public class DBHealthIndicatorTest {

    private AlivenessDao alivenessDao;
    private DBHealthIndicator dbHealthIndicator;

    @Before
    public void setUp() {
        alivenessDao = Mockito.mock(AlivenessDao.class);
        dbHealthIndicator = new DBHealthIndicator(alivenessDao);
    }

    @Test
    public void getHealthInfo_WhenDbIsAlive_ReturnsHealthIndicationOK() {
        Mockito.when(alivenessDao.isAlive()).thenReturn(true);

        HealthIndication healthInfo = dbHealthIndicator.getHealthInfo();

        assertEquals(HttpStatus.SC_OK, healthInfo.getStatus());
        assertTrue(healthInfo.getText().contains("OK"));
    }

    @Test
    public void getHealthInfo_WhenDbIsNotAlive_ReturnsHealthIndicationUnavailable() {
        Mockito.when(alivenessDao.isAlive()).thenReturn(false);

        HealthIndication healthInfo = dbHealthIndicator.getHealthInfo();

        assertEquals(HttpStatus.SC_SERVICE_UNAVAILABLE, healthInfo.getStatus());
        assertTrue(healthInfo.getText().contains("Unavailable"));
    }
}