package dk.nsp.idsas.operations.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.service.CleanupService;
import dk.nsp.idsas.service.SaltService;
import dk.nsp.idsas.util.health.CleanupHealthIndicator;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;

import java.time.Duration;
import java.time.Period;

@Configuration
@PropertySources({
        @PropertySource("classpath:/META-INF/maven/dk.nsp.idsas/idsas-operations-web/pom.properties"),
        @PropertySource(value = "idsas-operations.properties", encoding = "ISO-8859-1")
        // Bemærkning: I Java 8 er properties altid i ISO-8859-1, men fremadrettet skulle de gerne kunne være UTF-8.
})
public class WebSetup {
    @Value("${idsas.cleanup.max.duration:PT15S}")
    private String cleanupMaxDuration;

    @Value("${idsas.cleanup.max.retention:PT5Y}")
    private String cleanupMaxRetention;

    @Value("${idsas.cleanup.batch.size:10000}")
    private int cleanupBatchSize;

    @Bean
    public SaltService saltService(SaltDao saltDao, WhitelistDao whitelistDao) {
        return new SaltService(saltDao, whitelistDao);
    }

    @Bean
    public CleanupService cleanupService(BlurringDao blurringDao) {
        return new CleanupService(blurringDao, Duration.parse(cleanupMaxDuration), cleanupBatchSize, Period.parse(cleanupMaxRetention).getYears());
    }

    @Bean
    public CleanupHealthIndicator cleanupHealthIndicator(CleanupService cleanupService) {
        return new CleanupHealthIndicator(cleanupService);
    }
}
