package dk.nsp.idsas.setup;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.PatientIdSaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.http.PoolingHttpClientFactory;
import dk.nsp.idsas.integrations.personinformation.PersonInformationClient;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.health.PatientIdSaltHealthIndicator;
import dk.nsp.idsas.util.health.PersonInformationHealthIndicator;
import org.apache.http.client.HttpClient;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.PropertySource;
import org.springframework.context.annotation.PropertySources;

import java.time.Duration;

@Configuration
@PropertySources({
        @PropertySource("classpath:/META-INF/maven/dk.nsp.idsas/idsas-registration-web/pom.properties"),
        @PropertySource(value = "idsas-registration.properties", encoding = "ISO-8859-1")
        // Bemærkning: I Java 8 er properties altid i ISO-8859-1, men fremadrettet skulle de gerne kunne være UTF-8.
})
public class WebSetup {

    @Value("${personinformation.url}")
    private String personInformationUrl;

    @Value("${personinformation.errorcount.duration:PT10M}")
    private String personInformationErrorCountDuration;

    @Value("${personinformation.error.tolerance:0}")
    private int personInformationErrorTolerance;

    @Bean
    public PoolingHttpClientFactory poolingHttpClientFactory() {
        return new PoolingHttpClientFactory();
    }

    @Bean
    public PersonInformationClient personInformationClient(PoolingHttpClientFactory poolingHttpClientFactory) {
        HttpClient httpClient = poolingHttpClientFactory.getClient();
        return new PersonInformationClient(httpClient, personInformationUrl, Duration.parse(personInformationErrorCountDuration));
    }

    @Bean
    public BlurringService blurringService(WhitelistDao whitelistDao, BlurringDao blurringDao, PatientIdSaltDao patientIdSaltDao,
                                           AuditLogger auditLogger, PersonInformationClient personInformationClient) {
        return new BlurringService(whitelistDao, blurringDao, patientIdSaltDao, auditLogger, personInformationClient);
    }

    @Bean
    public PersonInformationHealthIndicator personInformationHealthIndicator(PersonInformationClient personInformationClient) {
        return new PersonInformationHealthIndicator(personInformationClient, personInformationErrorTolerance);
    }

    @Bean
    public PatientIdSaltHealthIndicator saltHealthIndicator(PatientIdSaltDao saltDao) {
        return new PatientIdSaltHealthIndicator(saltDao);
    }
}
