package dk.nsp.idsas.ws._20230601;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.service.BlurringService;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.CreateBlurringRequestType;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.IdsasRegistrationPortType;
import org.apache.cxf.annotations.SchemaValidation;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;
import java.time.ZonedDateTime;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasRegistration", urlPatterns={"/20230601/registration"})
@WebService(serviceName = "IdsasRegistration", wsdlLocation = "wsdl/20230601/idsas-registration.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasRegistrationPort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
public class IdsasRegistrationWS extends IdsasAbstractWS implements IdsasRegistrationPortType {
    private static final Logger logger = Logger.getLogger(IdsasRegistrationWS.class);

    @Autowired
    private IdsasActorProvider idsasActorProvider;

    @Autowired
    private BlurringService blurringService;

    @Override
    public String createBlurring(CreateBlurringRequestType createBlurringRequest) {
        logger.trace("Enter create blurring");

        try {
            IdsasActor actor = idsasActorProvider.getActor();
            String actorId = actor.getActorId();
            String actorIdType = actor.getActorIdType();
            String patientIdType = createBlurringRequest.getPatientId().getClassification().value();
            String patientId = createBlurringRequest.getPatientId().getId();
            ZonedDateTime endDateTime = createBlurringRequest.getEndDateTime().toGregorianCalendar().toZonedDateTime();

            blurringService.createBlurring(actorId, actorIdType, patientId, patientIdType, endDateTime);
        } catch (IdsasException e) {
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage());
        } catch (Exception e) {
            logger.error("Failed to create blurring", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error");
        }

        return "OK";
    }
}
