package dk.nsp.idsas.ws._20230601;

import dk.nsp.idsas.security.IdsasActorProvider;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.service.SaltService;
import dk.nsp.idsas.util.IdsasException;
import dk.nsp.idsas.util.SoapFaultGenerator;
import dk.nsp.idsas.ws.IdsasAbstractWS;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.GetCurrentSaltRequest;
import dk.sundhedsdatastyrelsen.identitetssloering._2023._06._01.IdsasSaltPortType;
import org.apache.cxf.annotations.SchemaValidation;
import org.apache.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;

import javax.jws.WebService;
import javax.servlet.annotation.WebServlet;
import javax.xml.ws.BindingType;

import static javax.xml.ws.soap.SOAPBinding.SOAP11HTTP_BINDING;

@WebServlet(name="IdsasSalt", urlPatterns={"/20230601/salt"})
@WebService(serviceName = "IdsasSalt", wsdlLocation = "wsdl/20230601/idsas-salt.wsdl",
        targetNamespace = "http://sundhedsdatastyrelsen.dk/identitetssloering/2023/06/01/",
        portName = "IdsasSaltPort")
@BindingType(value = SOAP11HTTP_BINDING)
@SchemaValidation(type = SchemaValidation.SchemaValidationType.BOTH)
public class IdsasSaltWS extends IdsasAbstractWS implements IdsasSaltPortType {
    private static final Logger logger = Logger.getLogger(IdsasSaltWS.class);

    @Autowired
    private IdsasActorProvider idsasActorProvider;

    @Autowired
    private SaltService saltService;

    @Override
    public byte[] getCurrentSalt(GetCurrentSaltRequest getCurrentSaltRequest) {
        logger.trace("Enter get current salt");

        try {
            IdsasActor actor = idsasActorProvider.getActor();
            return saltService.getCurrentSalt(actor);
        } catch (IdsasException e) {
            throw SoapFaultGenerator.generateSOAPFault(e.isClient(), e.getMessage());
        } catch (Exception e) {
            logger.error("Failed to get current salt", e);
            throw SoapFaultGenerator.generateSOAPFault(false, "Internal server error");
        }
    }
}
