package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.PatientIdSaltDao;
import dk.nsp.idsas.dao.entity.BlurringEntity;
import dk.nsp.idsas.dto.OrganizationIdDto;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.AuditLogger;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.HashingUtil;

import java.util.List;
import java.util.stream.Collectors;

public class LookupService {
    private final BlurringDao blurringDao;
    private final PatientIdSaltDao patientIdSaltDao;
    private final AuditLogger auditLogger;
    private final String allowedClientSystemName;

    public LookupService(BlurringDao blurringDao, PatientIdSaltDao patientIdSaltDao,
                         AuditLogger auditLogger, String allowedClientSystemName) {
        this.blurringDao = blurringDao;
        this.patientIdSaltDao = patientIdSaltDao;
        this.auditLogger = auditLogger;
        this.allowedClientSystemName = allowedClientSystemName;
    }

    public List<OrganizationIdDto> getActiveBlurrings(IdsasActor actor, String patientId, String patientIdType) throws IdsasServiceException {
        // Validering af, at actor er en systembruger
        if (!actor.getActorType().equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actor.getActorType().name()), true);
        }

        // Validering af, at systembrugeren har et clientName med den korrekte værdi
        if (!actor.getClientSystemName().equals(allowedClientSystemName)) {
            throw new IdsasServiceException(ErrorMessages.clientSystemNameError(actor.getClientSystemName(), allowedClientSystemName), true);
        }

        // Hente entiteter
        byte[] patientIdSalt = patientIdSaltDao.getLatestSalt();
        String hashedPatientId = HashingUtil.createPatientIdHash(patientId, patientIdSalt);
        List<BlurringEntity> entityList = blurringDao.getActiveBlurrings(hashedPatientId, patientIdType);

        // Mappe entiteter til liste af id'ere
        List<OrganizationIdDto> orgIdList = entityList.stream().map(OrganizationIdDto.mapper).collect(Collectors.toList());

        // Audit logning
        auditLogger.logGetBlurredOrganisations(patientId);

        return orgIdList;
    }
}

