package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.SaltDao;
import dk.nsp.idsas.dao.WhitelistDao;
import dk.nsp.idsas.security.model.IdsasActor;
import dk.nsp.idsas.util.ErrorMessages;
import dk.nsp.idsas.util.WhitelistType;

import java.security.SecureRandom;

public class SaltService {
    private final SaltDao saltDao;
    private final WhitelistDao whitelistDao;
    private final SecureRandom secureRandom;

    public SaltService(SaltDao saltDao, WhitelistDao whitelistDao) {
        this(saltDao, whitelistDao, new SecureRandom());
    }

    public SaltService(SaltDao saltDao, WhitelistDao whitelistDao, SecureRandom secureRandom) {
        this.saltDao = saltDao;
        this.whitelistDao = whitelistDao;
        this.secureRandom = secureRandom;
    }

    public byte[] getCurrentSalt(IdsasActor actor) throws IdsasServiceException {
        validateSystemUser(actor.getActorType());
        validateClientWhitelisted(actor.getClientKey());
        return saltDao.getLatestSalt();
    }

    public void renewSalt() {
        byte[] salt = new byte[16];
        secureRandom.nextBytes(salt);
        saltDao.insert(salt);
    }

    private void validateSystemUser(IdsasActor.IdsasActorType actorType) throws IdsasServiceException {
        if (!actorType.equals(IdsasActor.IdsasActorType.SYSTEM)) {
            throw new IdsasServiceException(ErrorMessages.userTypeNotSupported(actorType.name()), true);
        }
    }

    private void validateClientWhitelisted(String clientKey) throws IdsasServiceException {
        if (!whitelistDao.isWhitelisted(clientKey, WhitelistType.SALT)) {
            throw new IdsasServiceException(ErrorMessages.clientKeyWhitelistError(clientKey, WhitelistType.SALT), true);
        }
    }
}
