package dk.nsp.idsas.service;

import dk.nsp.idsas.dao.BlurringDao;
import dk.nsp.idsas.dao.CleanupJobStatus;
import org.junit.Before;
import org.junit.Test;
import org.mockito.Mockito;
import org.mockito.internal.stubbing.answers.AnswersWithDelay;

import java.time.Duration;

import static org.junit.Assert.assertEquals;

public class CleanupServiceTest {
    private BlurringDao blurringDao;

    @Before
    public void setUp() {
        blurringDao = Mockito.mock(BlurringDao.class);
    }

    @Test
    public void doCleanup_WhenExecutionTimeIsExceeded_statusIsUnfinished() {
        // Arrange

        // Max execution time is 1 seconds, batch size 150
        CleanupService cleanupService = new CleanupService(blurringDao, Duration.ofSeconds(1), 150, 5);
        // 10000 rows to delete in total
        Mockito.when(blurringDao.getInactiveBlurringsCount()).thenReturn(10000);
        // Each back takes 500 milliseconds
        Mockito.when(blurringDao.deleteRowsInBatch(Mockito.anyInt())).thenAnswer(new AnswersWithDelay(500, invocation -> invocation.getArgument(0)));

        // Act
        cleanupService.cleanUpBlurrings();

        // Assert
        CleanupJobStatus status = cleanupService.getStatus();
        assertEquals(300, status.rowsDeleted);
        assertEquals(CleanupJobStatus.ExecutionStatus.NOT_FINISHED, status.finished);
    }

    @Test
    public void doCleanup_WhenAllRowsAreDeleted_statusIsFinished() {
        // Arrange

        // Max execution time is 100 seconds, batch size 100
        CleanupService cleanupService = new CleanupService(blurringDao, Duration.ofSeconds(100), 100, 5);
        // 210 rows to delete in total
        Mockito.when(blurringDao.getInactiveBlurringsCount()).thenReturn(210);
        Mockito.when(blurringDao.deleteRowsInBatch(100))
                .thenReturn(100)
                .thenReturn(100)
                .thenReturn(10);

        // Act
        cleanupService.cleanUpBlurrings();

        // Assert
        CleanupJobStatus status = cleanupService.getStatus();
        assertEquals(210, status.rowsDeleted);
        assertEquals(CleanupJobStatus.ExecutionStatus.FINISHED, status.finished);
    }

}
